import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

extension StringExtension on String {
  /// Check email validation
  bool validateEmail() => RegExp(
          r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+")
      .hasMatch(this);

  /// Check phone validation
  bool validatePhone() => RegExp(r'(^(?:[+0]9)?[0-9]{10,12}$)').hasMatch(this);

  /// Check URL validation
  bool validateURL() {
    return RegExp(
            r'^((?:.|\n)*?)((http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)([-A-Z0-9.]+)(/[-A-Z0-9+&@#/%=~_|!:,.;]*)?(\?[A-Z0-9+&@#/%=~_|!:‌​,.;]*)?)')
        .hasMatch(this);
  }

  /// Returns true if given String is null or isEmpty
  bool get isEmptyOrNull => this == null || isEmpty;

  // Check null string, return given value if null
  String validate({String value = ''}) => this ?? value;

  /// Capitalize given String
  String capitalizeFirstLetter() => (validate().length >= 1)
      ? (substring(0, 1).toUpperCase() + substring(1).toLowerCase())
      : validate();

  /// Return true if given String is Digit
  bool isDigit(String s) {
    if (s.isEmpty) {
      return false;
    }
    if (s.length > 1) {
      for (var r in s.runes) {
        if (r ^ 0x30 > 9) {
          return false;
        }
      }
      return true;
    } else {
      return s.runes.first ^ 0x30 <= 9;
    }
  }

  /// Print given String
  log() {
    var dateTime = DateTime.now();
    print('${dateTime.hour}:${dateTime.minute}:${dateTime.second} $this');
  }

  // Copy String to Clipboard
  copyToClipboard() async {
    if (this == null || this.length <= 0) return;
    await Clipboard.setData(ClipboardData(text: this));
  }

  Text text({
    String defaultValue = '',
    TextStyle style,
    int maxLines,
    TextOverflow overflow = TextOverflow.ellipsis,
    TextAlign textAlign,
    bool softWrap,
    Locale locale,
    Key key,
    String semanticsLabel,
    StrutStyle strutStyle,
    TextDirection textDirection,
    double textScaleFactor,
    TextWidthBasis textWidthBasis,
  }) {
    return Text(
      this.validate(value: defaultValue),
      style: style,
      maxLines: maxLines,
      overflow: overflow,
      textAlign: textAlign,
      softWrap: softWrap,
      locale: locale,
      key: key,
      semanticsLabel: semanticsLabel,
      strutStyle: strutStyle,
      textDirection: textDirection,
      textScaleFactor: textScaleFactor,
      textWidthBasis: textWidthBasis,
    );
  }
}
